# glider-gun-builderv2.1.py
# TODO:  see if specialcases can be included in the regular template folder and the same dictionary lookup
# TODO:  recheck, e.g., the p00618 series, make sure it really needs to be customized.
# TODO:  read unmodified README from glider_guns_master folder, instead of copy-pasting into the script
# 
# DONE:  added a "specialcases" folder, check there before building
# DONE:  added INSTRUCTIONS text below
# DONE:  fixed orientation of p358, p363, and overtrimming problem in the p392s
# DONE:  fixed bad p436 gun IDs
# DONE:  moved template definitions to a "template" folder
# DONE:  use unmodified README
# DONE:  standardized on looking for "alt" definitions and using those instead of adjusted template
#
# INSTRUCTIONS
#
# This script is intended for use with the pattern archive at https://github.com/ceebo/glider_guns .
# Based on the variable-period guns in the "variable" folder, it will build guns of specific periods,
#   following the information found in README.md.
# To make this possible, it is necessary to mark each variable gun to show
#   1) where input signals should be added (yellow state 5, often a Herschel or glider but not always)
#   2) what part of the gun should be adjusted (white state 3, marking all cells that should move)
#   3) where the output of the gun should be expected (red state 4, should be on a bounding box edge)
#
# These template guns are saved in a folder named "template".
# The filenames of these template guns include some extra information:
#  -- the direction of adjustment for the movable part of the gun -- DX,DY;
#  -- the number of ticks to advance after placing input signals, to reach the output location
# This last is necessary because the input location is assumed to be in the stationary part
#   of the gun, along with the output.  In other words, input and output locations are not
#   changed when the gun is adjusted to a new period.  So in some cases, if the active signal
#   is in fact somewhere in the adjustable part of the gun when a glider escapes from the
#   bounding box, then it is necessary to "rewind" that signal by N ticks until it is in the
#   stationary part, then give OFFSET=N in the filename:
#
#   p{5-digit period with leading zeroes},DX,DY,OFFSET.rle
#
# In some situations, the early adjustments to a gun contain pieces that are too close to each other
#   that have to be custom-welded to avoid catastrophic explosions.  This must be done separately
#   for each individual situation, with the resulting gun saved as a separate template.  Filename:
#
#   p000{period}alt{adjustment_distance},0,0,offset.rle
#
# Notice that no DX or DY is needed, because with welded components no adjustment is possible.
#
# In some cases no offset is needed, but it can be used in a welded gun if necessary.
#
# The adjustability only works for simple linear displacements.  All guns with complex adjustability
#   must have a template pattern custom-adjusted for each required spacing.
#
# In rare cases, a gun with multiple signals in a loop will require a slightly different (larger)
#   shape than the base gun with only a single signal, to avoid conflicts between the signals.
#   In this case, a pattern should be added to the "specialcases" subfolder of the template folder.


import golly as g
import os, sys
from os import listdir
from os.path import isfile, join

# __file__ not available in Golly?
# scriptpath = os.path.dirname(os.path.abspath( __file__ ))
scriptpath =  os.path.realpath(os.path.dirname(sys.argv[0]))
outfolder = os.path.join(scriptpath, "guns")
infolder = os.path.join(scriptpath, "template")
specialfolder = os.path.join(scriptpath,"template", "specialcases")
fixedfolder = os.path.join(scriptpath,"glider_guns-master", "fixed")
vargunfolder = os.path.join(scriptpath,"glider_guns-master", "variable")

onlyfiles = [f for f in listdir(infolder) if isfile(join(infolder, f))]
baseguns=dict()
for file in onlyfiles:
  name, dx, dy, offset = file[:-4].split(",")  # remove '.rle'
  baseguns[name]=[g.load(join(infolder, file)),int(dx), int(dy), int(offset)]
specialguns=dict()
specialfiles = [f for f in listdir(specialfolder) if isfile(join(specialfolder, f))]
for file in specialfiles:  # these are not adjustable guns and are saved in final form,
                           # so no need for dx, dy, or offset
  name, dx, dy, offset = file[:-4].split(",")  # remove '.rle'
  specialguns[name]=[g.load(join(specialfolder, file)),int(dx), int(dy), int(offset)]
g.addlayer()

readme="14 (61308, 'fixed')\n15 (4004, 'fixed')\n16 (31152, 'fixed')\n17 (39990, 'fixed')\n18 (16377, 'fixed')\n19 (34953, 'fixed')\n20 (2800, 'fixed')\n21 (28258, 'fixed')\n22 (1125, 'fixed')\n23 (2312, 'fixed')\n24 (1376, 'fixed')\n25 (15587, 'fixed')\n26 (13734, 'fixed')\n27 (21717, 'fixed')\n28 (20292, 'fixed')\n29 (21160, 'fixed')\n30 (324, 'fixed')\n31 (15065, 'fixed')\n32 (14520, 'fixed')\n33 (15367, 'fixed')\n34 (14356, 'fixed')\n35 (16875, 'fixed')\n36 (2401, 'fixed')\n37 (9075, 'fixed')\n38 (16016, 'fixed')\n39 (7320, 'fixed')\n40 (2940, 'fixed')\n41 (10416, 'fixed')\n42 (7345, 'fixed')\n43 (11008, 'fixed')\n44 (1218, 'fixed')\n45 (1920, 'fixed')\n46 (784, 'fixed')\n47 (3538, 'fixed')\n48 (1794, 'fixed')\n49 (8892, 'fixed')\n50 (3276, 'fixed')\n51 (7980, 'fixed')\n52 (4232, 'fixed')\n53 (9856, 'fixed')\n54 (8614, 'fixed')\n55 (7875, 'fixed')\n56 (7497, 'fixed')\n57 (7503, 'fixed')\n58 (9900, 'fixed')\n59 (10250, 'fixed')\n60 (1053, 'fixed')\n61 (10332, 'fixed')\n62 (10824, 'fixed')\n63 (8208, 'fixed')\n64 (4355, 'fixed')\n65 (8976, 'fixed')\n66 (2295, 'fixed')\n67 (4968, 'fixed')\n68 (6960, 'fixed')\n69 (5390, 'fixed')\n70 (7812, 'fixed')\n71 (10830, 'fixed')\n72 (1330, 'fixed')\n73 (11128, 'fixed')\n74 (3630, 'fixed')\n75 (4278, 'fixed')\n76 (7930, 'fixed')\n77 (7254, 'fixed')\n78 (3248, 'p00358_4')\n79 (4160, 'p00363_4')\n80 (3500, 'fixed')\n81 (3819, 'p00405osc3_0')\n82 (4392, 'p00436_7')\n83 (4316, 'p00482_2')\n84 (4060, 'p00448_7')\n85 (3953, 'p00385osc5_5')\n86 (3843, 'p00358_9')\n87 (3800, 'fixed')\n88 (1512, 'fixed')\n89 (3127, 'p00356_0')\n90 (1110, 'fixed')\n91 (3240, 'p00356_1')\n92 (1150, 'fixed')\n93 (3355, 'p00356_2')\n94 (950, 'fixed')\n95 (3472, 'p00356_3')\n96 (2496, 'fixed')\n97 (3591, 'p00356_4')\n98 (3894, 'p00392_0')\n99 (3712, 'p00356_5')\n100 (4020, 'p00392_1')\n101 (3835, 'p00356_6')\n102 (4148, 'p00392_2')\n103 (3960, 'p00356_7')\n104 (861, 'fixed')\n105 (4087, 'p00356_8')\n106 (4410, 'p00392_4')\n107 (4216, 'p00356_9')\n108 (4544, 'p00392_5')\n109 (3510, 'p00436_0')\n110 (2600, 'fixed')\n111 (3630, 'p00436_1')\n112 (3213, 'p00448_0')\n113 (3752, 'p00436_2')\n114 (3328, 'p00448_1')\n115 (3876, 'p00436_3')\n116 (3445, 'p00448_2')\n117 (4002, 'p00436_4')\n118 (3564, 'p00448_3')\n119 (4130, 'p00436_5')\n120 (726, 'fixed')\n121 (3660, 'p00363_0')\n122 (2915, 'p00358_1')\n123 (3060, 'p00369osc3_0')\n124 (3355, 'p00356_2')\n125 (4526, 'p00436_8')\n126 (3410, 'p00378osc6_0')\n127 (4662, 'p00436_9')\n128 (3844, 'p00368osc4_2')\n129 (4032, 'p00363_3')\n130 (3248, 'p00358_4')\n131 (4758, 'p00345_6')\n132 (1806, 'fixed')\n133 (5082, 'p00436_12')\n134 (4588, 'p00448_11')\n135 (2205, 'fixed')\n136 (4148, 'p00392_2')\n137 (4422, 'p00363_6')\n138 (1683, 'fixed')\n139 (5184, 'p00345_9')\n140 (4087, 'p00356_8')\n141 (3782, 'fixed')\n142 (4345, 'p00418_1')\n143 (4189, 'fixed')\n144 (1155, 'fixed')\n145 (4720, 'p00564_2_p00145')\n146 (3968, 'p00358_10')\n147 (4071, 'fixed')\n148 (3630, 'p00436_1')\n149 (4838, 'p00564_4_p00149')\n150 (1575, 'fixed')\n151 (4897, 'p00564_5_p00151')\n152 (3328, 'p00448_1')\n153 (3740, 'p00443_2')\n154 (4340, 'fixed')\n155 (4026, 'p00465osc5_0')\n156 (1764, 'fixed')\n157 (5074, 'p00564_8_p00157')\n158 (4896, 'fixed')\n159 (5016, 'p00405osc3_9')\n160 (3685, 'p00448_4')\n161 (3685, 'p00483osc7_0')\n162 (4661, 'p00405osc9x2_0')\n163 (5278, 'p00652_0_p00163')\n164 (4392, 'p00436_7')\n165 (5256, 'p00455osc5_5')\n166 (3450, 'p00498_0')\n167 (5082, 'fixed')\n168 (4060, 'p00448_7')\n169 (4514, 'p00443_8')\n170 (5183, 'p00358_19')\n171 (4824, 'p00465osc3_6')\n172 (4800, 'p00436_10')\n173 (5568, 'p00652_5_p00173')\n174 (3816, 'p00498_3')\n175 (3816, 'p00350osc5_0')\n176 (1936, 'fixed')\n177 (4928, 'p00443_11')\n178 (3127, 'p00356_0')\n179 (2808, 'p00358_0')\n180 (1320, 'fixed')\n181 (5800, 'p00652_9_p00181')\n182 (3240, 'p00356_1')\n183 (2915, 'p00358_1')\n184 (1085, 'fixed')\n185 (4800, 'p00515osc5_5')\n186 (3355, 'p00356_2')\n187 (3024, 'p00358_2')\n188 (1586, 'fixed')\n189 (5183, 'p00567osc7_0')\n190 (3472, 'p00356_3')\n191 (3135, 'p00358_3')\n192 (2646, 'fixed')\n193 (5220, 'p00571_1_p00193')\n194 (3591, 'p00356_4')\n195 (3248, 'p00358_4')\n196 (3894, 'p00392_0')\n197 (5460, 'p00559_4')\n198 (2964, 'fixed')\n199 (3363, 'p00358_5')\n200 (3763, 'fixed')\n201 (4602, 'p00531osc3_9_p00201')\n202 (3835, 'p00356_6')\n203 (3480, 'p00358_6')\n204 (3965, 'p00408osc6_0')\n205 (5175, 'p00410osc5_0')\n206 (3960, 'p00356_7')\n207 (3528, 'p00414osc3_0')\n208 (1394, 'fixed')\n209 (4212, 'p00418_0')\n210 (1375, 'fixed')\n211 (3720, 'p00358_8')\n212 (4410, 'p00392_4')\n213 (3894, 'p00426osc3_0')\n214 (4216, 'p00356_9')\n215 (3843, 'p00358_9')\n216 (1764, 'fixed')\n217 (4480, 'p00418_2')\n218 (3510, 'p00436_0')\n219 (3894, 'p00414osc3_3')\n220 (2610, 'fixed')\n221 (4617, 'p00418_3')\n222 (3630, 'p00436_1')\n223 (4095, 'p00358_11')\n224 (3213, 'p00448_0')\n225 (4278, 'p00426osc3_3')\n226 (3752, 'p00436_2')\n227 (4224, 'p00358_12')\n228 (3328, 'p00448_1')\n229 (4897, 'p00418_5')\n230 (2336, 'fixed')\n231 (4278, 'p00414osc3_6')\n232 (3445, 'p00448_2')\n233 (5040, 'p00418_6')\n234 (3726, 'fixed')\n235 (4488, 'p00358_14')\n236 (3564, 'p00448_3')\n237 (4680, 'p00426osc3_6')\n238 (4130, 'p00436_5')\n239 (4623, 'p00358_15')\n240 (1155, 'fixed')\n241 (4050, 'p00482_0')\n242 (4260, 'p00436_6')\n243 (4560, 'p00405osc9x3_0')\n244 (3136, 'fixed')\n245 (4182, 'p00482_1')\n246 (1496, 'fixed')\n247 (4899, 'p00358_17')\n248 (3575, 'p00356x2_2')\n249 (3450, 'p00498_0')\n250 (4526, 'p00436_8')\n251 (5040, 'p00358_18')\n252 (4060, 'p00448_7')\n253 (3570, 'p00498_1')\n254 (4216, 'fixed')\n255 (5002, 'p00385osc15x3_5')\n256 (2401, 'fixed')\n257 (3692, 'p00498_2')\n258 (4800, 'p00436_10')\n259 (4130, 'p00518osc7_0_p00259')\n260 (3819, 'fixed')\n261 (3816, 'p00498_3')\n262 (4940, 'p00436_11')\n263 (5041, 'fixed')\n264 (1932, 'fixed')\n265 (3942, 'p00498_4')\n266 (5082, 'p00436_12')\n267 (4284, 'p00356x3_0')\n268 (4588, 'p00448_11')\n269 (4070, 'p00498_5')\n270 (1980, 'fixed')\n271 (5032, 'p00534_1')\n272 (4712, 'p00392osc16x2_2')\n273 (4200, 'p00498_6')\n274 (5175, 'fixed')\n275 (4307, 'p00550osc5_0_p00275')\n276 (1815, 'fixed')\n277 (4332, 'p00498_7')\n278 (4650, 'p00556_0')\n279 (4425, 'p00558osc9_0_p00279')\n280 (4331, 'p00356x2_8')\n281 (4466, 'p00498_8')\n282 (1700, 'fixed')\n283 (5467, 'p00534_4')\n284 (4320, 'fixed')\n285 (4543, 'p00380osc15x3_0')\n286 (4661, 'p00564_1_p00286')\n287 (4543, 'p00518osc7_7_p00287')\n288 (2040, 'fixed')\n289 (4740, 'p00498_10')\n290 (4720, 'p00564_2_p00290')\n291 (4824, 'p00356x3_4')\n292 (5110, 'p00358x2_10')\n293 (4880, 'p00498_11')\n294 (4437, 'p00482x3_1')\n295 (4602, 'p00550osc5_5_p00295')\n296 (3795, 'p00436x2_1')\n297 (4964, 'p00356x3_5')\n298 (4838, 'p00564_4_p00298')\n299 (5106, 'p00590_1')\n300 (1575, 'fixed')\n301 (5159, 'fixed')\n302 (4897, 'p00564_5_p00302')\n303 (5106, 'p00356x3_6')\n304 (4158, 'p00456osc16x2_0')\n305 (5312, 'p00498_14')\n306 (2550, 'fixed')\n307 (5244, 'p00590_3')\n308 (3256, 'fixed')\n309 (5250, 'p00356x3_7')\n310 (5015, 'p00564_7_p00310')\n311 (5313, 'p00590_4')\n312 (1650, 'fixed')\n313 (5610, 'p00498_16')\n314 (5074, 'p00564_8_p00314')\n315 (4897, 'p00550osc5_10_p00315')\n316 (5605, 'p00608_3_p00316')\n317 (5704, 'p00618_2_p00317')\n318 (5133, 'p00564_9_p00318')\n319 (5530, 'p00590_6')\n320 (5376, 'p00480x4_0')\n321 (5544, 'p00356x3_9')\n322 (2397, 'fixed')\n323 (5680, 'p00590_7')\n324 (5723, 'p00640_1')\n325 (5828, 'p00618_4_p00325')\n326 (5278, 'p00652_0_p00326')\n327 (4158, 'p00436x3_0')\n328 (4575, 'p00436x2_7')\n329 (5890, 'p00618_5_p00329')\n330 (2352, 'fixed')\n331 (5544, 'p00662_0')\n332 (4368, 'p00482x2_2')\n333 (4290, 'p00436x3_1')\n334 (5394, 'p00652_2_p00334')\n335 (5280, 'p00550osc5_15_p00335')\n336 (4422, 'p00448x3_0')\n337 (6014, 'p00618_7_p00337')\n338 (5452, 'p00652_3_p00338')\n339 (4424, 'p00436x3_2')\n340 (5959, 'p00640_5')\n341 (5609, 'fixed')\n342 (4556, 'p00448x3_1')\n343 (4484, 'p00686_0')\n344 (4992, 'p00436x2_10')\n345 (3850, 'p00345x3_0')\n346 (5568, 'p00652_5_p00346')\n347 (4620, 'p00686_1')\n348 (4692, 'p00448x3_2')\n349 (6048, 'p00698_0')\n350 (3816, 'p00350osc5_0')\n351 (4698, 'p00436x3_4')\n352 (2538, 'fixed')\n353 (3087, 'fixed')\n354 (4830, 'p00448x3_3')\n355 (4898, 'p00686_3')\n356 (3127, 'p00356_0')\n357 (4838, 'p00436x3_5')\n358 (2808, 'p00358_0')\n359 (5040, 'p00686_4')\n360 (1518, 'fixed')\n361 (4218, 'p00345_2')\n362 (5800, 'p00652_9_p00362')\n363 (3660, 'p00363_0')\n364 (3240, 'p00356_1')\n365 (5727, 'p00714_2')\n366 (2915, 'p00358_1')\n367 (5330, 'p00686_6')\n368 (1440, 'fixed')\n369 (3060, 'p00369osc3_0')\n370 (6018, 'p00652_11_p00370')\n371 (3782, 'p00363_1')\n372 (3355, 'p00356_2')\n373 (5734, 'p00746_0_p00373')\n374 (3024, 'p00358_2')\n375 (5270, 'p00436x3_8')\n376 (1804, 'fixed')\n377 (4484, 'p00345_4')\n378 (3410, 'p00378osc6_0')\n379 (3906, 'p00363_2')\n380 (3472, 'p00356_3')\n381 (5412, 'p00762_0_p00381')\n382 (3135, 'p00358_3')\n383 (5780, 'p00758_1_p00383')\n384 (3588, 'fixed')\n385 (3348, 'p00385osc5_0')\n386 (6237, 'p00740_4_p00386')\n387 (4032, 'p00363_3')\n388 (3591, 'p00356_4')\n389 (5544, 'p00762_2_p00389')\n390 (2970, 'fixed')\n391 (5916, 'p00758_3_p00391')\n392 (3894, 'p00392_0')\n393 (3402, 'p00369osc3_3')\n394 (6565, 'p00740_6_p00394')\n395 (4160, 'p00363_4')\n396 (2714, 'fixed')\n397 (5762, 'p00762_4_p00397')\n398 (3363, 'p00358_5')\n399 (5200, 'p00399osc7x3_0')\n400 (3763, 'fixed')\n401 (4898, 'p00345_7')\n402 (3584, 'p00402osc6_0')\n403 (4290, 'p00363_5')\n404 (3835, 'p00356_6')\n405 (3009, 'fixed')\n406 (3480, 'p00358_6')\n407 (3886, 'fixed')\n408 (3965, 'p00408osc6_0')\n409 (5040, 'p00345_8')\n410 (5175, 'p00410osc5_0')\n411 (4422, 'p00363_6')\n412 (3960, 'p00356_7')\n413 (4080, 'p00413osc7_0')\n414 (2597, 'fixed')\n415 (6510, 'p00726_13_p00415')\n416 (1998, 'fixed')\n417 (3762, 'p00369osc3_6')\n418 (4212, 'p00418_0')\n419 (4556, 'p00363_7')\n420 (2448, 'fixed')\n421 (3960, 'fixed')\n422 (3720, 'p00358_8')\n423 (4095, 'p00423osc9_0')\n424 (4410, 'p00392_4')\n425 (3953, 'p00385osc5_5')\n426 (3894, 'p00426osc3_0')\n427 (4692, 'p00363_8')\n428 (4216, 'p00356_9')\n429 (4189, 'fixed')\n430 (3038, 'fixed')\n431 (6596, 'p00862_0_p00431')\n432 (2160, 'fixed')\n433 (5478, 'p00345_11')\n434 (4480, 'p00418_2')\n435 (4830, 'p00363_9')\n436 (3510, 'p00436_0')\n437 (6720, 'p00874_0_p00437')\n438 (3894, 'p00414osc3_3')\n439 (4200, 'fixed')\n440 (2744, 'fixed')\n441 (4071, 'fixed')\n442 (4617, 'p00418_3')\n443 (3498, 'p00443_0')\n444 (3630, 'p00436_1')\n445 (4428, 'p00445_0')\n446 (4095, 'p00358_11')\n447 (6490, 'p00886_1_p00447')\n448 (3213, 'p00448_0')\n449 (5022, 'p00441_1')\n450 (2294, 'fixed')\n451 (3618, 'p00443_1')\n452 (3752, 'p00436_2')\n453 (4565, 'p00445_1')\n454 (4224, 'p00358_12')\n455 (4556, 'p00455osc5_0')\n456 (3328, 'p00448_1')\n457 (3953, 'fixed')\n458 (4897, 'p00418_5')\n459 (3740, 'p00443_2')\n460 (3312, 'fixed')\n461 (4704, 'p00445_2')\n462 (4278, 'p00414osc3_6')\n463 (6555, 'p00926_0')\n464 (3445, 'p00448_2')\n465 (4026, 'p00465osc3_0')\n466 (5040, 'p00418_6')\n467 (3864, 'p00443_3')\n468 (2610, 'fixed')\n469 (4845, 'p00445_3')\n470 (3776, 'fixed')\n471 (4818, 'fixed')\n472 (3564, 'p00448_3')\n473 (5313, 'p00457_2')\n474 (4680, 'p00426osc3_6')\n475 (3990, 'p00443_4')\n476 (4130, 'p00436_5')\n477 (4988, 'p00445_4')\n478 (4623, 'p00358_15')\n479 (7227, 'p00926_4')\n480 (1680, 'fixed')\n481 (5460, 'p00457_3')\n482 (4050, 'p00482_0')\n483 (3685, 'p00483osc7_0')\n484 (4260, 'p00436_6')\n485 (5015, 'p00485_0')\n486 (4680, 'p00414osc3_9')\n487 (7480, 'p00942_4_p00487')\n488 (3136, 'fixed')\n489 (4416, 'p00465osc3_3')\n490 (4182, 'p00482_1')\n491 (4248, 'p00443_6')\n492 (2080, 'fixed')\n493 (5037, 'fixed')\n494 (4899, 'p00358_17')\n495 (5256, 'p00455osc5_5')\n496 (3933, 'p00448_6')\n497 (4615, 'p00497osc7_0')\n498 (3450, 'p00498_0')\n499 (4380, 'p00443_7')\n500 (4526, 'p00436_8')\n501 (5082, 'fixed')\n502 (5040, 'p00358_18')\n503 (7524, 'p00886_15_p00503')\n504 (3025, 'fixed')\n505 (4686, 'p00465osc5_5')\n506 (3570, 'p00498_1')\n507 (4071, 'p00507osc3_0')\n508 (4216, 'fixed')\n509 (5456, 'p00485_3')\n510 (2107, 'fixed')\n511 (5372, 'fixed')\n512 (3294, 'fixed')\n513 (4824, 'p00465osc3_6')\n514 (3692, 'p00498_2')\n515 (4130, 'p00515osc5_0')\n516 (4800, 'p00436_10')\n517 (4810, 'fixed')\n518 (4130, 'p00518osc7_0')\n519 (6588, 'p00652x3_5_p00519')\n520 (2944, 'fixed')\n521 (5616, 'p00505_2')\n522 (3816, 'p00498_3')\n523 (4788, 'p00443_10')\n524 (4940, 'p00436_11')\n525 (3710, 'p00350osc5x3_0')\n526 (5041, 'fixed')\n527 (5112, 'fixed')\n528 (2244, 'fixed')\n529 (5767, 'p00505_3')\n530 (3942, 'p00498_4')\n531 (4130, 'p00531osc3_0')\n532 (5082, 'p00436_12')\n533 (5313, 'fixed')\n534 (4130, 'p00534osc6_0')\n535 (4828, 'p00535osc5_0')\n536 (4588, 'p00448_11')\n537 (3648, 'p00358x3_0')\n538 (4070, 'p00498_5')\n539 (4588, 'p00483osc7_7')\n540 (2088, 'fixed')\n541 (4928, 'fixed')\n542 (5032, 'p00534_1')\n543 (6580, 'p00708x3_2_p00543')\n544 (4725, 'p00448_12')\n545 (5396, 'p00465osc5_10')\n546 (4200, 'p00498_6')\n547 (5214, 'p00443_13')\n548 (5175, 'fixed')\n549 (3770, 'p00358x3_1')\n550 (4307, 'p00550osc5_0')\n551 (6408, 'fixed')\n552 (1470, 'fixed')\n553 (5074, 'p00553osc7_0')\n554 (4332, 'p00498_7')\n555 (4307, 'p00531osc3_3')\n556 (4650, 'p00556_0')\n557 (5254, 'p00549_1')\n558 (4307, 'p00534osc6_3')\n559 (5180, 'p00559_0')\n560 (5005, 'p00448_14')\n561 (3894, 'p00358x3_2')\n562 (4466, 'p00498_8')\n563 (5508, 'p00443_15')\n564 (2107, 'fixed')\n565 (5325, 'p00549_2')\n566 (5467, 'p00534_4')\n567 (5183, 'p00567osc7_0')\n568 (4320, 'fixed')\n569 (5106, 'p00561_1')\n570 (4543, 'p00380osc15x3_0')\n571 (5160, 'p00571_0')\n572 (4661, 'p00564_1')\n573 (4020, 'p00358x3_3')\n574 (4543, 'p00518osc7_7')\n575 (5183, 'p00535osc5_5')\n576 (2160, 'fixed')\n577 (5175, 'p00561_2')\n578 (4740, 'p00498_10')\n579 (4484, 'p00531osc3_6')\n580 (4720, 'p00564_2')\n581 (5168, 'p00581_0')\n582 (4484, 'p00534osc6_6')\n583 (5390, 'p00559_3')\n584 (4836, 'p00568osc8_2')\n585 (4148, 'p00358x3_4')\n586 (4880, 'p00498_11')\n587 (5280, 'p00571_2')\n588 (4779, 'p00564_3')\n589 (5313, 'p00581_1')\n590 (4602, 'p00550osc5_5')\n591 (5460, 'p00559_4')\n592 (4015, 'p00436x4_1')\n593 (5313, 'p00561_4')\n594 (4964, 'p00356x3_5')\n595 (5340, 'p00571_3')\n596 (4838, 'p00564_4')\n597 (4278, 'p00358x3_5')\n598 (5022, 'fixed')\n599 (5530, 'p00559_5')\n600 (1785, 'fixed')\n601 (4784, 'p00601_0')\n602 (5159, 'fixed')\n603 (4602, 'p00531osc3_9')\n604 (4897, 'p00564_5')\n605 (5609, 'p00581_3')\n606 (4661, 'p00534osc6_9')\n607 (5680, 'p00559_6')\n608 (4830, 'p00448x4_1')\n609 (4410, 'p00358x3_6')\n610 (5312, 'p00498_14')\n611 (5460, 'p00571_5')\n612 (3570, 'fixed')\n613 (5760, 'p00581_4')\n614 (5244, 'p00590_3')\n615 (5336, 'p00615_0')\n616 (3478, 'fixed')\n617 (5076, 'p00601_2')\n618 (5192, 'p00618osc6_0')\n619 (5520, 'p00571_6')\n620 (5015, 'p00564_7')\n621 (4544, 'p00358x3_7')\n622 (5313, 'p00590_4')\n623 (5394, 'p00615_1')\n624 (1908, 'fixed')\n625 (5225, 'p00601_3')\n626 (5610, 'p00498_16')\n627 (4104, 'p00418x3_0')\n628 (5074, 'p00564_8')\n629 (6068, 'p00581_6')\n630 (2700, 'fixed')\n631 (5452, 'p00615_2')\n632 (4872, 'p00616osc8_2')\n633 (4680, 'p00358x3_8')\n634 (5704, 'p00618_2')\n635 (5640, 'p00571_8')\n636 (5133, 'p00564_9')\n637 (6225, 'p00581_7')\n638 (5530, 'p00590_6')\n639 (4235, 'p00418x3_1')\n640 (5376, 'p00480x4_0')\n641 (5529, 'p00601_5')\n642 (5251, 'p00642osc6_0')\n643 (5307, 'p00643_0')\n644 (2688, 'fixed')\n645 (4662, 'p00430osc15x3_0')\n646 (5680, 'p00590_7')\n647 (5568, 'p00615_4')\n648 (2900, 'fixed')\n649 (5640, 'p00649_0')\n650 (5828, 'p00618_4')\n651 (4368, 'p00418x3_2')\n652 (5278, 'p00652_0')\n653 (6391, 'p00653_0')\n654 (4158, 'p00436x3_0')\n655 (5626, 'p00615_5')\n656 (4819, 'p00436x4_7')\n657 (4958, 'p00358x3_10')\n658 (3008, 'fixed')\n659 (5429, 'p00643_2')\n660 (2700, 'fixed')\n661 (6468, 'p00653_1')\n662 (5544, 'p00662_0')\n663 (4503, 'p00418x3_3')\n664 (4420, 'p00482x4_2')\n665 (5940, 'p00665osc7_0')\n666 (4290, 'p00436x3_1')\n667 (5490, 'p00643_3')\n668 (5394, 'p00652_2')\n669 (5100, 'p00358x3_11')\n670 (5192, 'p00550osc5_15')\n671 (5742, 'p00615_7')\n672 (4422, 'p00448x3_0')\n673 (6111, 'p00649_3')\n674 (6014, 'p00618_7')\n675 (4640, 'p00418x3_4')\n676 (5452, 'p00652_3')\n677 (6622, 'p00653_3')\n678 (4424, 'p00436x3_2')\n679 (5800, 'p00615_8')\n680 (5208, 'p00616osc8_8')\n681 (4980, 'p00681_0')\n682 (5609, 'fixed')\n683 (4920, 'p00683_0')\n684 (4556, 'p00448x3_1')\n685 (5340, 'p00645osc5_5')\n686 (4484, 'p00686_0')\n687 (4779, 'p00418x3_5')\n688 (5248, 'p00436x4_10')\n689 (5124, 'p00681_1')\n690 (1248, 'fixed')\n691 (4980, 'p00683_1')\n692 (5568, 'p00652_5')\n693 (5208, 'p00693_0')\n694 (4620, 'p00686_1')\n695 (6018, 'p00615_10')\n696 (4692, 'p00448x3_2')\n697 (5185, 'p00681_2')\n698 (6048, 'p00698_0')\n699 (4920, 'p00418x3_6')\n700 (4717, 'p00350osc5x2_0')\n701 (5270, 'p00693_1')\n702 (4698, 'p00436x3_4')\n703 (6180, 'p00615_11')\n704 (2784, 'fixed')\n705 (5332, 'p00681_3')\n706 (3905, 'fixed')\n707 (5185, 'p00683_3')\n708 (4830, 'p00448x3_3')\n709 (5418, 'p00693_2')\n710 (4898, 'p00686_3')\n711 (5063, 'p00418x3_7')\n712 (3339, 'p00356x2_0')\n713 (5394, 'p00681_4')\n714 (4838, 'p00436x3_5')\n715 (5332, 'p00683_4')\n716 (3780, 'p00358x2_0')\n717 (5481, 'p00693_3')\n718 (5040, 'p00686_4')\n719 (5808, 'p00719_0')\n720 (1680, 'fixed')\n721 (5395, 'p00721_0')\n722 (4959, 'p00345x2_2')\n723 (4300, 'p00482x3_0')\n724 (5504, 'p00724_0')\n725 (5632, 'p00693_4')\n726 (4757, 'p00363x2_0')\n727 (5874, 'p00719_1')\n728 (2640, 'fixed')\n729 (5460, 'p00721_1')\n730 (5727, 'p00714_2')\n731 (5544, 'p00683_6')\n732 (3904, 'p00358x2_1')\n733 (5696, 'p00693_5')\n734 (5330, 'p00686_6')\n735 (4182, 'p00490osc15x3_0')\n736 (1710, 'fixed')\n737 (5525, 'p00721_2')\n738 (2009, 'fixed')\n739 (5607, 'p00683_7')\n740 (5605, 'p00740_0')\n741 (5850, 'p00693_6')\n742 (4896, 'p00363x2_1')\n743 (5984, 'p00743_0')\n744 (3575, 'p00356x2_2')\n745 (5590, 'p00721_3')\n746 (5734, 'p00746_0')\n747 (4576, 'p00482x3_2')\n748 (4030, 'p00358x2_2')\n749 (5915, 'p00693_7')\n750 (3564, 'fixed')\n751 (6052, 'p00743_1')\n752 (2340, 'fixed')\n753 (5655, 'p00721_4')\n754 (5251, 'p00345x2_4')\n755 (5824, 'p00683_9')\n756 (5402, 'p00448x3_7')\n757 (6052, 'p00741_2')\n758 (5037, 'p00363x2_2')\n759 (4717, 'p00482x3_3')\n760 (3696, 'p00356x2_3')\n761 (5720, 'p00721_5')\n762 (5168, 'fixed')\n763 (5980, 'p00683_10')\n764 (4158, 'p00358x2_3')\n765 (6120, 'p00741_3')\n766 (5695, 'p00726_5')\n767 (6188, 'p00743_3')\n768 (3430, 'fixed')\n769 (5874, 'p00721_6')\n770 (5400, 'p00345x2_5')\n771 (4860, 'p00482x3_4')\n772 (6164, 'p00724_6')\n773 (6188, 'p00741_4')\n774 (5180, 'p00363x2_3')\n775 (6348, 'p00743_4')\n776 (3819, 'p00356x2_4')\n777 (5355, 'p00518osc7x3_0_p00777')\n778 (5544, 'p00762_2')\n779 (6204, 'p00683_12')\n780 (2989, 'fixed')\n781 (6348, 'p00741_5')\n782 (5829, 'p00726_7')\n783 (5005, 'p00482x3_5')\n784 (4588, 'p00392osc16x2_0')\n785 (6097, 'p00721_8')\n786 (5307, 'p00345x6_6')\n787 (6270, 'p00683_13')\n788 (6392, 'p00724_8')\n789 (6035, 'fixed')\n790 (5325, 'p00363x2_4')\n791 (6580, 'p00743_6')\n792 (2688, 'fixed')\n793 (6164, 'p00721_9')\n794 (5676, 'p00762_4')\n795 (4872, 'p00490osc15x3_5')\n796 (4420, 'p00358x2_5')\n797 (6580, 'p00741_7')\n798 (5874, 'p00436x3_12')\n799 (6650, 'p00743_7')\n800 (4800, 'p00392x2_1')\n801 (6048, 'p00534x3_0')\n802 (5704, 'p00345x2_7')\n803 (6499, 'p00683_15')\n804 (4620, 'p00402osc6x2_0')\n805 (5508, 'fixed')\n806 (5472, 'p00363x2_5')\n807 (5301, 'p00482x3_7')\n808 (4071, 'p00356x2_6')\n809 (6392, 'p00721_11')\n810 (2640, 'fixed')\n811 (6664, 'p00683_16')\n812 (4554, 'p00358x2_6')\n813 (6205, 'p00534x3_1')\n814 (6279, 'p00726_11')\n815 (6728, 'p00815_0')\n816 (4420, 'p00408osc48x2_0')\n817 (6555, 'p00721_12')\n818 (5859, 'p00345x2_8')\n819 (5452, 'p00482x3_8')\n820 (6806, 'p00410osc5x2_0')\n821 (6831, 'p00813_1')\n822 (5621, 'p00363x2_6')\n823 (6786, 'p00815_1')\n824 (4200, 'p00356x2_7')\n825 (5544, 'p00550osc5x3_0_p00825')\n826 (4800, 'p00413osc7x2_0')\n827 (5822, 'p00827_0')\n828 (3266, 'fixed')\n829 (6831, 'p00813_2')\n830 (6510, 'p00726_13')\n831 (5605, 'p00482x3_9')\n832 (1998, 'fixed')\n833 (6790, 'p00721_14')\n834 (5025, 'p00556x3_0')\n835 (5893, 'p00827_1')\n836 (4914, 'p00418x2_0')\n837 (5670, 'p00558osc9x3_0_p00837')\n838 (5772, 'p00363x2_7')\n839 (6902, 'p00815_3')\n840 (2108, 'fixed')\n841 (6222, 'p00841_0')\n842 (6256, 'p00762_10')\n843 (5760, 'p00482x3_10')\n844 (4828, 'p00358x2_8')\n845 (5880, 'fixed')\n846 (2322, 'fixed')\n847 (5727, 'p00847osc7_0')\n848 (4914, 'p00392osc16x2_4')\n849 (6386, 'p00841_1')\n850 (6175, 'p00345x2_10')\n851 (6048, 'p00827_3')\n852 (4740, 'p00426osc6x2_0')\n853 (6386, 'p00853_0')\n854 (5925, 'p00363x2_8')\n855 (5917, 'p00482x3_11')\n856 (4464, 'p00356x2_9')\n857 (6552, 'p00841_2')\n858 (5313, 'p00556x3_2')\n859 (6048, 'p00827_4')\n860 (4352, 'fixed')\n861 (5796, 'p00518osc7x3_7_p00861')\n862 (6596, 'p00862_0')\n863 (6720, 'p00863_0')\n864 (2430, 'fixed')\n865 (6300, 'fixed')\n866 (6336, 'p00345x2_11')\n867 (6076, 'p00482x3_12')\n868 (5208, 'p00418x2_2')\n869 (6720, 'p00853_2')\n870 (2852, 'fixed')\n871 (6784, 'p00863_1')\n872 (3672, 'p00436x2_0')\n873 (6344, 'p00582osc3x3_0')\n874 (6720, 'p00762_14')\n875 (6120, 'fixed')\n876 (5110, 'p00358x2_10')\n877 (6572, 'p00853_3')\n878 (5530, 'fixed')\n879 (6237, 'p00482x3_13')\n880 (3364, 'fixed')\n881 (6420, 'p00865_2')\n882 (4941, 'p00441x2_0')\n883 (6360, 'p00867_2')\n884 (5358, 'p00418x2_3')\n885 (5859, 'p00550osc5x3_5_p00885')\n886 (4891, 'p00443x2_0')\n887 (6864, 'p00879_1')\n888 (3795, 'p00436x2_1')\n889 (6480, 'p00865_3')\n890 (5580, 'p00445x2_0')\n891 (6364, 'p00827_8')\n892 (5254, 'p00358x2_11')\n893 (6696, 'p00853_5')\n894 (5760, 'p00556x3_5')\n895 (6720, 'p00895_0')\n896 (4550, 'p00448x2_0')\n897 (6314, 'p00590x3_1')\n898 (5084, 'p00441x2_1')\n899 (6480, 'p00867_4')\n900 (2257, 'fixed')\n901 (6758, 'p00853_6')\n902 (5032, 'p00443x2_1')\n903 (6264, 'p00847osc7_7')\n904 (3920, 'p00436x2_2')\n905 (6600, 'p00865_5')\n906 (5734, 'p00445x2_1')\n907 (6525, 'p00827_10')\n908 (5400, 'p00358x2_12')\n909 (6396, 'p00590x3_2')\n910 (5100, 'p00455osc5x2_0')\n911 (6840, 'p00895_2')\n912 (4158, 'p00456osc16x2_0')\n913 (6660, 'p00865_6')\n914 (5229, 'p00441x2_2')\n915 (6097, 'fixed')\n916 (5664, 'p00418x2_5')\n917 (6540, 'p00861osc7_7')\n918 (3472, 'fixed')\n919 (6900, 'p00895_3')\n920 (3312, 'fixed')\n921 (6478, 'p00590x3_3')\n922 (5890, 'p00445x2_2')\n923 (6660, 'p00867_7')\n924 (3922, 'fixed')\n925 (6600, 'p00845osc5_10')\n926 (6555, 'p00926_0')\n927 (6030, 'p00618x3_0_p00927')\n928 (4824, 'p00448x2_2')\n929 (6780, 'p00865_8')\n930 (3666, 'fixed')\n931 (6720, 'p00867_8')\n932 (5820, 'p00418x2_6')\n933 (6560, 'p00590x3_4')\n934 (5320, 'p00443x2_3')\n935 (7020, 'p00895_5')\n936 (2958, 'fixed')\n937 (6840, 'p00865_9')\n938 (5829, 'p00413osc7x2_7')\n939 (6097, 'p00618x3_1_p00939')\n940 (3840, 'fixed')\n941 (7068, 'p00853_11')\n942 (6205, 'fixed')\n943 (7080, 'p00895_6')\n944 (4424, 'p00456osc16x2_2')\n945 (6174, 'p00550osc5x3_10_p00945')\n946 (5525, 'p00441x2_4')\n947 (6840, 'p00867_10')\n948 (5525, 'p00426osc6x2_6')\n949 (7130, 'p00853_12')\n950 (5467, 'p00443x2_4')\n951 (6164, 'p00618x3_2_p00951')\n952 (4307, 'p00436x2_5')\n953 (6960, 'p00865_11')\n954 (5984, 'p00405osc9x2_9')\n955 (6875, 'p00915osc5_5')\n956 (5850, 'p00358x2_15')\n957 (6724, 'p00590x3_6')\n958 (6930, 'p00886_9')\n959 (6840, 'p00847osc7_14')\n960 (1938, 'fixed')\n961 (7020, 'p00865_12')\n962 (5676, 'p00441x2_5')\n963 (6231, 'p00618x3_3_p00963')\n964 (4100, 'p00482x2_0')\n965 (6900, 'p00845osc5_15')\n966 (3074, 'fixed')\n967 (7381, 'p00895_9')\n968 (4440, 'p00436x2_6')\n969 (6806, 'p00590x3_7')\n970 (6240, 'p00485x2_0')\n971 (7020, 'p00867_13')\n972 (6004, 'p00358x2_16')\n973 (6960, 'p00861osc7_14')\n974 (7168, 'p00886_11')\n975 (6298, 'fixed')\n976 (3584, 'fixed')\n977 (7140, 'p00865_14')\n978 (5829, 'p00441x2_6')\n979 (7080, 'p00867_14')\n980 (4233, 'p00482x2_1')\n981 (4840, 'p00436x9_0')\n982 (5767, 'p00443x2_6')\n983 (7626, 'p00895_11')\n984 (2080, 'fixed')\n985 (7320, 'p00865_15')\n986 (6402, 'p00485x2_1')\n987 (6365, 'p00618x3_5_p00987')\n988 (6160, 'p00358x2_17')\n989 (7440, 'p00853_17')\n990 (2898, 'fixed')\n991 (7688, 'p00895_12')\n992 (4930, 'p00488osc8x2_1')\n993 (5544, 'p00662x3_0')\n994 (5984, 'p00441x2_7')\n995 (7150, 'p00915osc5_10')\n996 (4368, 'p00482x2_2')\n997 (7502, 'p00853_18')\n998 (5920, 'p00443x2_7')\n999 (4895, 'p00436x9_1')"
readmelist = readme.split("\n")
warnings = ""
countbuilt = 0
countfixed = 0
countvar0 = 0
countconfirmed = 0
for item in readmelist:
  period, size, basegun = item.split(" ")
  basegun=basegun.replace("'","").replace(")","")
  bb, usebb = [],0
  if basegun=="fixed":
    g.show("[fixed]")
    countfixed+=1
    fixedname="p"+("0000"+period)[-5:]
    g.open(join(fixedfolder, fixedname+".rle"))
    bb=g.getrect()
    usebb=1
    g.setrule("LifeHistoryToLife")
    g.run(1)
    g.setrule("Life")
    g.save(os.path.join(outfolder,fixedname)+"_fixed.rle","rle")
    r=g.getrect()
  else:
    datalist = basegun.split("_")
    if len(datalist)==2:
      baseperiod, adjustment = datalist
    else:
      baseperiod, adjustment, finalperiod = datalist
    subperiod = 1 # this should probably really be called the "period multiplier" or some such -- it's for when you add a semi-Snark, etc.
    temp = baseperiod.split('x')
    if len(temp)==2:
      subperiod = int(temp[1])
    iperiod,  iadjustment = int(period), int(adjustment)
    if baseperiod+"alt"+adjustment in baseguns:
      baseperiod+="alt"+adjustment
      iadjustment=0 # weld has already been done at specific adjustment
  #  if baseperiod!="p00534": # "p00392osc16x2": # "p00601":
  #    continue  #  had a lot of trouble with period 601.  Ironic since I put that one together.
  #    ######################
    if baseperiod not in baseguns:
      if iadjustment==0:
        # we have no template pattern, but presumably we have the one gun in the variable folder, anyway
        countvar0+=1
        g.open(join(vargunfolder, baseperiod+".rle"))
        bb=g.getrect()
        usebb=1
        g.setrule("LifeHistoryToLife")
        g.run(1)
        g.setrule("Life")
        variablegunname="p"+("0000"+period)[-5:]
        # we would still have to fill in multiple signals for cases where that's needed
        #        (i.e., where len(temp)==2, where there's an 'x' in the name)
        flag="_NEEDS_TEMPLATE_"+basegun if len(temp)==2 else "" # these all actually have templates now (5/20/2018)
        g.save(os.path.join(outfolder,variablegunname)+"_variable0"+flag+".rle","rle")
      continue        # TODO:  adjust script to handle families with only one gun the same as template families
    else:  # there's a matching adjustable gun in the template folder.

      basepat, dx, dy, finaladjustment = specialguns[basegun] if basegun in specialguns else baseguns[baseperiod]
      fixedpat,movablepat,output,signal=[],[],[],[]
      for i in range(0,len(basepat)-1,3):
        if basepat[i+2]==1:
          fixedpat+=basepat[i:i+3]
        elif basepat[i+2]==3:
          movablepat+=basepat[i:i+2]+[1]
        elif basepat[i+2]==4:
          output+=basepat[i:i+2]+[0]
        elif basepat[i+2]==5:
          signal+=basepat[i:i+2]+[1]
        else:
          g.exit("Don't know that color: "+str(basepat[i+2]))
      if len(fixedpat)%2 == 0: fixedpat+=[0]
      if len(movablepat)%2 == 0: movablepat+=[0]
      if len(output)%2 == 0: output+=[0]
      if len(signal)%2 == 0: signal+=[0]
      g.new(basegun)
      g.putcells(fixedpat)
      g.putcells(movablepat, iadjustment*dx, iadjustment*dy)
      done = 0
      g.setrule("Life")
      r=g.getrect()
      while done == 0:
        # g.note(str([iperiod,subperiod]))#####################
        g.putcells(signal)
        g.run(iperiod/subperiod)
        g.fit()
        g.update()
        if g.getcell(signal[0],signal[1])==1: done=1
    #  g.note("Running final adjustment " +str(finaladjustment))
      g.run(finaladjustment)
    #  g.note("Running lotsa times " + str(iperiod*100/subperiod)+" "+str(subperiod))
      g.run(100*iperiod/subperiod)
      # TODO: put the next three copy/paste code chunks with outputmatchflag into a function.
      outputmatchflag=1
      for outcell in range(0,len(output),3):
        if g.getcell(output[outcell],output[outcell+1])!=1:
#          g.note(str([output,outcell,outcell+1, output[outcell],output[outcell+1], g.getcell(output[outcell],output[outcell+1])]))
          outputmatchflag=0
          break
      if subperiod>1 and outputmatchflag!=1:
        countdown=subperiod
        g.setgen("0")
        while countdown>=0:
          countdown-=1
          g.run(iperiod/subperiod)
          # g.update()
          # g.show("Counting down: "+str(countdown))
          outputmatchflag=1
          for outcell in range(0,len(output),3):
            if g.getcell(output[outcell],output[outcell+1])!=1:
              outputmatchflag=0
              break
          if outputmatchflag==1:
            break
      outputmatchflag=1
      for outcell in range(0,len(output),3):
        if g.getcell(output[outcell],output[outcell+1])!=1:
          outputmatchflag=0
          break
      if outputmatchflag!=1:
        g.exit("Looks like something went wrong at period " + period + ".  Output is not showing up after any reasonable number of subperiod cycles.")
      g.select(r)
      g.clear(1)
      g.select([])
      g.setgen("0")
      g.show("Done!  "+ os.path.join(outfolder,str(period)))
      g.save(os.path.join(outfolder,"p"+str(iperiod+100000)[1:]+"_"+basegun+".rle"),"rle")
      countbuilt+=1

  #check we get the expected size
  if usebb==1: r = bb  # guns not built from template should use their LifeHistory bounds
  size = int(size.replace(",","").replace("(",""))
  if r[2]<r[3]:
    warnings+="\nbad orientation " + item
  if r[2] * r[3] != size:
    warnings+="\nbad size " + item + ": really "+str(r[2]*r[3])
  maxx = r[0] + r[2]
  maxy = r[1] + r[3]

  #bbox should increase in the south or east
  g.run(1)
  r = g.getrect()
  if r[0] + r[2] <= maxx and r[1] + r[3] <= maxy:
    warnings+="\nno escape " + item

outmsg = "Done." if warnings=="" else "Done.  Errors:\n" + warnings
g.note(outmsg + "\nGuns built=" + str(countbuilt)+", fixed="+str(countfixed)+", from var folder="+str(countvar0)+"\n -- total=" +str(countbuilt+countfixed+countvar0))
g.setclipstr(outmsg + "\nGuns built=" + str(countbuilt)+", fixed="+str(countfixed)+", from var folder="+str(countvar0)+"\n -- total=" +str(countbuilt+countfixed+countvar0))